# !/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 4560 $
$LastChangedDate:: 2021-07-13 16:56:47 +0200#$

This file is part of SyMSpace.

Copyright (c) 2020, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""

import asyncio
import logging
import json_tricks
import numpy as numpy
import math
import os
import random
import signal
import sys
import time


from neuropil import NeuropilNode, np_token, np_message
from _neuropil import lib as np

logging.basicConfig(filename='square_server.log', format='%(asctime)s %(name)-15.15s:%(levelname)-7.7s : %(message)s', datefmt='%Y:%m:%d %H:%M:%S', level=logging.DEBUG)

logger = logging.getLogger('square_server')
logger.addHandler(logging.StreamHandler(sys.stdout))
logger.setLevel(logging.DEBUG)


settings = {
    'namespace'                : b'at:lcm:symspace',
    'a_square_b_square_subject': b'at:lcm:symspace:a_square_b_square',
    'datamodel_in'             : None,
    'a_square_b_square_replies': b'at:lcm:symspace:a_square_b_square:reply',
    'datamodel_out'            : None,

    # technical connectivity details
    # 'bootstrap_node'           : '*:udp4:localhost:2345',
    'bootstrap_node'           : '*:udp4:demo.neuropil.io:3400',
}

class SquareServer(NeuropilNode):
    """
    example neuropil node to calculate the square of two big numbers using numpy
    result will be send back on the same subject
    """
    def __init__(self, bootstrap_node, subject):
        super(SquareServer, self).__init__(port=random.randint(1025, 10250), proto=b'pas4', log_file="./neuropil_square_server.log", n_threads=0)

        self.subject = subject

        self.set_authorize_cb(self.authorize)
        self.set_authenticate_cb(self.authenticate)

        logger.info("requesting to join {bt_host} network ...".format(bt_host = bootstrap_node))
        self.join(bootstrap_node)
        while (False == self.has_joined()):
            logger.info("waiting to join the network ...")
            time.sleep(0.1)

        logger.info("successfully joined the network as: {node_hash}".format(node_hash=str(self.get_fingerprint())))
        self.bootstrap_node = bootstrap_node

        properties = self.get_mx_properties(settings['a_square_b_square_replies'])
        properties.ackmode = np.NP_MX_ACK_CLIENT
        # properties.intent_update_after = 10.0
        properties.max_retry = 3
        properties.apply()

        properties = self.get_mx_properties(self.subject)
        properties.ackmode = np.NP_MX_ACK_CLIENT
        # properties.intent_update_after = 10.0
        properties.max_retry = 3
        properties.apply()

        logger.info(f'waiting on {self.subject} for incoming data')
        self.set_receive_cb(self.subject, self.run_fnc)
        logger.info(f'setup of squareserver done: {self.bootstrap_node}')

    @staticmethod
    def authorize(self, token:np_token):
        logger.info('authorized access for token {token}'.format(token=token.get_fingerprint()))
        return True

    @staticmethod
    def authenticate(self, token:np_token):
        logger.info('authenticated access for token {token}'.format(token=token.get_fingerprint()))
        return True

    def handle_reconnect(self):
        self.join(self.bootstrap_node)

    def handle_shutdown(self, signum, frame):
        self.shutdown()

    async def start(self):
        # i = 0
        status = np.np_ok
        logger.info(f"Starting event loop")
        try:
            while(status == np.np_ok):
                # logger.debug("running server event loop: {iteration} | {hash}".format(iteration=i, hash=self.get_fingerprint()))
                status = self.run(0)
                # i = i+1
                await asyncio.sleep(0)
        except NeuropilException:
            logger.info("stopping neuropil main event loop")
        # finally:
        #     logger.info(f"stopping event loop")
        #     self.shutdown()        

    @staticmethod
    def run_fnc(self, input_message:np_message):
        logger.info('received new input data for subject {subject}'.format(subject=self.subject))

        logger.info("Processing data ...")
        input_data = json_tricks.loads(input_message.raw().decode("utf-8"))

        logger.info("Calculating ...")
        a = input_data['a']
        b = input_data['b']
        c = numpy.sqrt(a**2 + b**2)
        logger.info("Calculating done")

        output_data = {'c': c}

        logger.info("Sending result ...")
        self.send(settings['a_square_b_square_replies'], bytes(json_tricks.dumps(output_data), encoding='UTF-8'))

        return True



if __name__ == "__main__":

    logger.info("starting square server module")    

    app = SquareServer(settings['bootstrap_node'], settings['a_square_b_square_subject'])

    loop = asyncio.get_event_loop()

    logger.info("installing shutdown hook")        
    loop.add_signal_handler(signal.SIGINT, app.handle_shutdown)
    loop.add_signal_handler(signal.SIGTERM, app.handle_shutdown)
    # signal.signal(signal.SIGINT, app.handle_shutdown)
    # signal.signal(signal.SIGTERM, app.handle_shutdown)
    # signal.signal(signal.SIGKILL, app.handle_shutdown)
    # signal.signal(signal.SIGHUP, app.handle_reconnect)

    # app.start()

    loop.run_until_complete(app.start())
