#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 5784 $
$LastChangedDate:: 2022-04-21 10:13:47 +0200#$

This file is part of SyMSpace.

Copyright (c) 2019, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import os
import time
import subprocess
import open_webservice
from symspace.parampythonfnc import ParamPythonFnc
from symspace.decorators import remotedockercompute

'''
with a settings dict it is possible to inject a certain behaviour into the client.
e.g. the datamodel could be defined using pydantic models, and thus a verification step can be implemented easily
'''
settings = {
    # data dependent configuration / information
    'namespace': b'at:lcm:symspace',
    'subject': b'at:lcm:symspace:a_square_b_square',
    'datamodel_in': None,
    'replies': b'at:lcm:symspace:a_square_b_square:reply',
    'datamodel_out': None,

    # technical connectivity details
    # 'bootstrap_node': '*:udp4:localhost:2345',
    'bootstrap_node': '*:udp4:demo.neuropil.io:3400',

}


# @remotedockercompute('heroku_neurophil_showcase')
@remotedockercompute('neuropil_demo:latest')
class FncHerokuClientNeuropil(ParamPythonFnc):
    """ Heroku Client Python Function
    
    SyMSpace Heroku Client Template class,
    which shows the basic implementation of a Heroku Client Python function.

    """

    def importVars(self):
        """Description of the import variables

        Returns a list of tuples, which defines the import variables.
        Each tuple consists of multiple entries in the following order:

        (Name : string, Description : string, Unit : string, Optional : bool, Type / Dimension : string)

        """
        return [('a', 'Adjacent leg', 'mm', False, 'Numeric / [1 x 1]'),
                ('b', 'Opposite leg', 'mm', False, 'Numeric / [1 x 1]'),
                ('runDemoServer', 'Start a local server for testing/demonstration', '', False, 'Boolean')]

    def exportVars(self):
        """Description of the export variables

        Returns a list of tuples, which defines the export variables.
        Each tuple consists of multiple entries in the following order:

        (Name : string, Description : string, Unit : string, Optional : bool, Type / Dimension : string)

        """
        return [('c', 'Hypotenuse', 'mm', False, 'Numeric / [1 x 1]')]

    def getAdditionalPackages(self) -> list:
        pkg_list = [os.path.dirname(open_webservice.__file__)]
        return pkg_list

    def run(self, run_args):
        """Run method of the function, which calculates the result

        Parameters
        ----------
        run_args : dict
                Arguments of caller (Control flags)
                {RemoteCompute=false, RunSim=false, Optimize=false}

        Returns
        -------
        status : int
            If status == 0 the function regenerated successfully
            Else the function update is not completed OR failed

        """
        '''import must be hidden in the function'''
        from open_webservice.neuropil.client import NeuropilClient

        '''import variables from ports'''
        d = self.importValues()

        run_demo_server = d['runDemoServer']

        if run_demo_server:
            self.starDemoServer()

        input_data = dict(a=d['a'], b=d['b'])

        app = NeuropilClient(settings)
        output_data = app.run_loop(input_data)
        d['c'] = output_data['c']

        '''export variables to ports'''
        self.exportValues(d)

        return 0

    @staticmethod
    def starDemoServer():
        """Start a demo server, which runs at the same location as the client"""

        '''import must be hidden in the function'''
        import open_webservice.neuropil.server as server

        process = subprocess.Popen('neuropil_controller')
        while process.poll() is not None:
            print('Waiting for "neuropil_controller" to start')
            time.sleep(1)

        process = subprocess.Popen(['python3', server.__file__])
        while process.poll() is not None:
            print('Waiting for "server" to start')
            time.sleep(1)
