#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.5

$LastChangedRevision: 5146 $
$LastChangedDate:: 2021-11-03 08:22:53 +0100#$

This file is part of SyMSpace/ComponentSpace.

Copyright (c) 2018, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""

import os
import time
from componentspace.geometry_component import GeometryComponent
from symspace.osutils.pathutils import Paths
import shutil
import componentspace.osutils.fileutils as fu
from symspace.wrapper.symwrapper import SymProject, SymContainer, SymField, SymDirectory, SymFile, SymMaterial

class FreeCADGeometryComponentBuilder(GeometryComponent):
    """
    Create geometry component definition project
    """

    def __init__(self, geo_file, geo_class='FreeCAD'):
        """
        Init function

        Parameters
        ----------
        geo_file : str
            Geometry file to be converted.

        geo_class : str
            Class of geometry.
        """

        self.geo_file = geo_file
        self.geo_class = geo_class
        self.name = os.path.basename(geo_file).replace('.mop', '_' + geo_class)
        self.path = os.path.dirname(geo_file) + '\\' + self.name
        self.sym_mdl = None     # SymProject.load(os.path.join(self.path, self.name))
        self.parCAD = None      # Was a python function fnc_freecad_... (wrapper function)

        self.settings = []      # MAG2D, THERM2D, ...
        self.part = []          # Filepath to Python file containing the geometry file

        self.lstCADparam = []
        self.lstMATparam = []

    def createDirectory(self):
        """
        Creates a directory with the name of the geometry and the component class
        Copies directories /rst, /help, and /doc

        """
        # Remove CAD directory
        if os.path.exists(self.path):
            shutil.rmtree(self.path, onerror=fu.on_rm_error)
            # wait until OS is finished
            cnt = 0
            while os.path.exists(self.path):
                time.sleep(1)
                cnt = cnt + 1
                if cnt > 3:
                    raise ValueError('Cannot remove path!')

        # Create CAD directory
        if not os.path.exists(self.path):
            os.mkdir(self.path)

        fname = os.path.join(self.path, self.name + '.mop')
        if os.path.isfile(fname):
            os.remove(fname)
        if os.path.exists(fname + '.Files'):
            shutil.rmtree(fname + '.Files')

        geo_path = os.path.dirname(self.geo_file)
        try:
            fu.copyTree(os.path.join(geo_path, 'rst'), os.path.join(self.path, 'rst'))
        except:
            print('Directory \\rst already exists')

        # Copy src
        try:
            fu.copyTree(os.path.join(geo_path, 'src'), os.path.join(self.path, 'src'))
        except:
            print('Directory \\src already exists')

        try:
            fu.copyTree(os.path.join(geo_path, 'doc'), os.path.join(self.path, 'doc'))
        except:
            print('Directory \\doc already exists')
        # Copy index
        try:
            shutil.copy(os.path.join(geo_path, 'index.rst'), os.path.join(self.path, 'index.rst'))
        except:
            print('Cannot copy index.rst')

        # Copy CADLogo
        src_logo = os.path.join(Paths.getComponentSpaceRoot(), 'Part', 'resources', self.geo_class, 'CADLogo.png')
        dst_logo = os.path.join(self.path, 'rst')

        try:
            if os.path.isfile(os.path.join(dst_logo, 'CADLogo.png')):
                os.remove(os.path.join(dst_logo, 'CADLogo.png'))
            shutil.copy(src_logo, dst_logo)
        except:
            print('Cannot copy logo file')
            print(src_logo)
            print(os.path.join(dst_logo, 'CADLogo.png'))

        try:
            fu.copyTree(os.path.join(os.path.dirname(self.geo_file), 'FreeCAD'), os.path.join(self.path, 'FreeCAD'))
        except:
            print('Directory \\FreeCAD already exists')

        # Save source as new file
        prj = SymProject.load(self.geo_file)
        prj.saveAs(os.path.join(self.path, self.name))

    def convertFunction(self):
        """
        Convert container to SymPythonFunction
        """

        self.sym_mdl = SymProject.load(os.path.join(self.path, self.name))
        self.parCAD = self.sym_mdl.getObject('Project').getParam()[0]

        # Add fields from 'Add' container (Settings, Par)
        pos = 1
        if self.sym_mdl[self.geo_class + '.Add'] is not None:
            for par in self.sym_mdl[self.geo_class + '.Add'].getParam():
                self.parCAD.addParam(par, pos)
                pos += 1

        self.parCAD.setImportAssignments({'Settings': 'Settings'})
        self.parCAD.setImportAssignments({'Part': 'Part'})

        # Copy _3DModel directory
        sdir = self.sym_mdl.getObject(self.geo_class + '.Files.3DModel').getData()
        dir_mdl = os.path.join(self.sym_mdl.getAllFilesDirectory(), '_3DModel')
        fu.copyTree(sdir, dir_mdl)

        # Add 3DModel to component
        model3D = SymDirectory('3DModel')
        self.sym_mdl.getObject('Files').addParam(model3D)
        model3D.setData(dir_mdl)

        # Copy FreeCAD version
        try:
            freeCADVersion = self.sym_mdl.getObject(self.geo_class + '.FreeCADVersion').getData()
        except:
            # TODO: Add info message
            freeCADVersion = '0.15'

        # Get Units
        units = ''
        for par in self.parCAD.getParam():
            if isinstance(par, SymField) and par.getUnit() != '':
                units = units + "\n            '{}': '{}',".format(par.getName(), par.getUnit())

        # Relocate FreeCAD Python files
        for par in self.sym_mdl.getParameters('SymFile'):
            ext = os.path.splitext(par.getData())[1]
            if ext == '.py':
                par.setData(os.path.join(dir_mdl, os.path.basename(par.getData())))

        # Set import assignments
        self.sym_mdl.regenerate()
        paramImportMap = {}
        for par in self.parCAD.getParam():
            if isinstance(par, SymField) and par.getType() == 'geo':
                paramImportMap[par.getName()] = par.getName()
        self.parCAD.setImportAssignments(paramImportMap)

        # Assign import params from import interface
        for imp in self.parCAD.getImports():
            if imp.getParamName() == '' and self.parCAD.getObject(imp.getVarName()) is not None:
                self.parCAD.setImportAssignments({imp.getVarName(): imp.getVarName()})

        # Regenerate, save and close the project
        self.sym_mdl.regenerate()
        self.sym_mdl.save()
        self.sym_mdl.close()

    def createComponent(self):
        """
        Create Component definition file

        """
        self.createDirectory()
        # self.createBuildScript()
        self.convertFunction()
        self.updateInfo()
        self.sym_mdl.save()