#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 3910 $
$LastChangedDate:: 2020-11-24 14:43:23 +0100#$

This file is part of SyMSpace.

Copyright (c) 2019, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import os
import sys
import pkgutil
import warnings
import symspace.osutils.processutils

try:
    from symspace.wrapper.symwrapper import SymField
    from symspace.wrapper.symwrapper import SymProject
except ImportError:
    SymField = None
    SymProject = None


def getPackageVersion(pkg_name: str) -> str:
    """
    Get the package version from the package name.

    Package has to be importable.

    If version contains __revision__
    this will be replaced by the svn revision of the package folder.

    Parameters
    ----------
    pkg_name :str
        name of the package e.g. lcm_motorbox.postprocessing

    Returns
    -------
    version :str
    """
    warnings.warn("Deprecated method", DeprecationWarning)

    pkg = __import__(pkg_name)
    pkg_path = os.path.dirname(pkg.__file__)
    version = pkg.__version__

    svn_revision, _ = symspace.osutils.processutils.getSVNRevision(pkg_path)
    version = version.replace('__revision__', svn_revision)
    return version


def getPackageRevision(pkg_name: str, src_path: str = None) -> str:
    """
    Get the package revision from the package name

    Parameters
    ----------
    pkg_name : str
    src_path : str
        Alternative way to find the package

    Returns
    -------
    revision : str
    """
    warnings.warn("Deprecated method", DeprecationWarning)

    if src_path is None:
        pkg = __import__(pkg_name)
        pkg_path = os.path.dirname(pkg.__file__)
    else:
        pkg_path = os.path.join(src_path, *pkg_name.split('.'))

    svn_revision, error = symspace.osutils.processutils.getSVNRevision(pkg_path)
    return svn_revision


def getPackageRevisons(start_path: str) -> dict:
    """
    Get a dictionary with all packages and their revisions.

    Parameters
    ----------
    start_path:str

    Returns
    -------
    d : dict
    """
    warnings.warn("Deprecated method", DeprecationWarning)

    d = dict()

    ''' Add Package path to search path '''
    if start_path not in sys.path:
        sys.path.append(start_path)

    for loader, pkg_name, is_pkg in pkgutil.walk_packages([start_path]):
        if is_pkg:
            svn_revision = getPackageRevision(pkg_name)
            d[pkg_name] = svn_revision
    return d


def getPackageVersions(start_path: str) -> dict:
    """
    Get a dictionary with all packages and their versions (altered by revision).

    Parameters
    ----------
    start_path:str

    Returns
    -------
    d : dict
    """
    d = dict()

    ''' Add Package path to search path '''
    if start_path not in sys.path:
        sys.path.append(start_path)

    for loader, pkg_name, is_pkg in pkgutil.walk_packages([start_path]):
        if is_pkg:
            version = getPackageVersion(pkg_name)
            d[pkg_name] = version
    return d


def findUsedPackages(prj, get_version: bool = False) -> dict:
    if SymField is None:
        raise ImportError('Unable to import SymField')
    if SymProject is None:
        raise ImportError('Unable to import SymProject')

    pkg_versions = dict()
    if prj.getObject('Packages') is not None:
        for param in prj.getObject('Packages').getParam():
            if isinstance(param, SymField):
                pkg_name = param.getData()
                if get_version:
                    pkg_versions[pkg_name] = getPackageVersion(pkg_name)
                else:
                    '''return dummy version to speed up the process'''
                    pkg_versions[pkg_name] = 0
    return pkg_versions


if __name__ == '__main__':
    from symspace.osutils.pathutils import Paths

    _pkg_path = os.path.join(Paths.getComponentSpaceRoot(), 'src', 'packages')
    print(getPackageRevisons(_pkg_path))
